using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Workflow.Runtime;

namespace SodaMachine
{
    public partial class Form1 : Form
    {
        // Our workflow runtime instance
        WorkflowRuntime _workflowRuntime = null;

        // Currently executing workflow instance (we'll only have
        // one).
        WorkflowInstance _workflowInstance = null;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Create an instance of the workflow runtime
            _workflowRuntime = WorkflowFactory.GetWorkflowRuntime();
            _workflowRuntime.WorkflowTerminated += new EventHandler<WorkflowTerminatedEventArgs>(workflowRuntime_WorkflowTerminated);
            _workflowRuntime.WorkflowCompleted += new EventHandler<WorkflowCompletedEventArgs>(workflowRuntime_WorkflowCompleted);
        }

        void workflowRuntime_WorkflowCompleted(object sender, WorkflowCompletedEventArgs e)
        {
            // Disconnect the event handlers
            SodaService.WorkflowSodaMachineDataService dataService = SodaService.WorkflowSodaMachineDataService.GetRegisteredWorkflowDataService();
            dataService.TotalChanged -= new EventHandler<SodaService.TotalChangedEventArgs>(dataService_TotalChanged);
            dataService.Dispense -= new EventHandler<SodaService.DispenseEventArgs>(dataService_Dispense);

            // Remove the data service
            SodaService.SodaMachineDataConnector dataConnector = (SodaService.SodaMachineDataConnector)_workflowRuntime.GetService(typeof(SodaService.SodaMachineDataConnector));
            _workflowRuntime.RemoveService(dataConnector);

            // Clear instance (for application termination purposes)
            _workflowInstance = null;
        }

        void workflowRuntime_WorkflowTerminated(object sender, WorkflowTerminatedEventArgs e)
        {
            // Disconnect the event handlers
            SodaService.WorkflowSodaMachineDataService dataService = SodaService.WorkflowSodaMachineDataService.GetRegisteredWorkflowDataService();
            dataService.TotalChanged -= new EventHandler<SodaService.TotalChangedEventArgs>(dataService_TotalChanged);
            dataService.Dispense -= new EventHandler<SodaService.DispenseEventArgs>(dataService_Dispense);

            // Remove the data service
            SodaService.SodaMachineDataConnector dataConnector = (SodaService.SodaMachineDataConnector)_workflowRuntime.GetService(typeof(SodaService.SodaMachineDataConnector));
            _workflowRuntime.RemoveService(dataConnector);

            // Clear instance (for application termination purposes)
            _workflowInstance = null;

            // Some error...
            if (e.Exception.Message != "User reset requested")
                MessageBox.Show(String.Format("The soda machine workflow was terminated! Error: {0}", e.Exception.Message));
        }

        void dataService_Dispense(object sender, SodaService.DispenseEventArgs e)
        {
            IAsyncResult result = this.BeginInvoke(
                new EventHandler(
                       delegate
                       {
                           // Activate the soda buttons...
                           cmdBottle1.Enabled = true;
                           cmdBottle2.Enabled = true;
                           cmdBottle3.Enabled = true;
                           cmdBottle4.Enabled = true;
                           cmdBottle5.Enabled = true;

                           // Deactivate the coin buttons
                           cmd5.Enabled = false;
                           cmd10.Enabled = false;
                           cmd25.Enabled = false;

                           // Display the total
                           lcdDisplay1.Value = e.FinalTotal;
                       } // delegate
                    ), null, null
            ); // BeginInvoke
            this.EndInvoke(result);
        }

        void dataService_TotalChanged(object sender, SodaService.TotalChangedEventArgs e)
        {
            IAsyncResult result = this.BeginInvoke(
                new EventHandler(
                       delegate
                       {
                           // Display the total
                           lcdDisplay1.Value = e.CurrentTotal;
                       } // delegate
                    ), null, null
            ); // BeginInvoke
            this.EndInvoke(result);
        }

        private void cmdExit_Click(object sender, EventArgs e)
        {
            // Just quit...
            Application.Exit();
        }

        private void cmdReset_Click(object sender, EventArgs e)
        {
            // Force the application to restart (since our workflow hit
            // a terminal condition, we must restart everything from
            // scratch).
            Application.Restart();
        }

        private void StartWorkflow()
        {
            _workflowInstance = _workflowRuntime.CreateWorkflow(typeof(SodaFlow.Workflow1));

            // Hook returned data event
            SodaService.WorkflowSodaMachineDataService dataService = SodaService.WorkflowSodaMachineDataService.CreateDataService(_workflowInstance.InstanceId, _workflowRuntime);
            dataService.TotalChanged += new EventHandler<SodaService.TotalChangedEventArgs>(dataService_TotalChanged);
            dataService.Dispense += new EventHandler<SodaService.DispenseEventArgs>(dataService_Dispense);

            // Start instance.
            _workflowInstance.Start();

            // Allow time for first state transition
            System.Threading.Thread.Sleep(200);
        }

        private void cmd5_Click(object sender, EventArgs e)
        {
            // If there is no workflow started, start one.
            SodaService.SodaMachineDataConnector dataConnector = null;
            if (_workflowInstance == null)
            {
                // Create instance.
                StartWorkflow();
            } // if

            // Deposit a quarter
            dataConnector = (SodaService.SodaMachineDataConnector)_workflowRuntime.GetService(typeof(SodaService.SodaMachineDataConnector));
            dataConnector.RaiseCoinInserted(_workflowInstance.InstanceId, 0.05m);
        }

        private void cmd10_Click(object sender, EventArgs e)
        {
            // If there is no workflow started, start one.
            SodaService.SodaMachineDataConnector dataConnector = null;
            if (_workflowInstance == null)
            {
                // Create instance.
                StartWorkflow();
            } // if

            // Deposit a quarter
            dataConnector = (SodaService.SodaMachineDataConnector)_workflowRuntime.GetService(typeof(SodaService.SodaMachineDataConnector));
            dataConnector.RaiseCoinInserted(_workflowInstance.InstanceId, 0.1m);
        }

        private void cmd25_Click(object sender, EventArgs e)
        {
            // If there is no workflow started, start one.
            SodaService.SodaMachineDataConnector dataConnector = null;
            if (_workflowInstance == null)
            {
                // Create instance.
                StartWorkflow();
            } // if

            // Deposit a quarter
            dataConnector = (SodaService.SodaMachineDataConnector)_workflowRuntime.GetService(typeof(SodaService.SodaMachineDataConnector));
            dataConnector.RaiseCoinInserted(_workflowInstance.InstanceId, 0.25m);
        }

        private void SodaDropped(object sender, EventArgs e)
        {
            // Show the soda label
            lblSoda.Visible = true;

            // Disable the soda buttons
            cmdBottle1.Enabled = false;
            cmdBottle2.Enabled = false;
            cmdBottle3.Enabled = false;
            cmdBottle4.Enabled = false;
            cmdBottle5.Enabled = false;

            // Notify the workflow
            SodaService.SodaMachineDataConnector dataConnector = (SodaService.SodaMachineDataConnector)_workflowRuntime.GetService(typeof(SodaService.SodaMachineDataConnector));
            dataConnector.RaiseItemSelected(_workflowInstance.InstanceId);
        }
    }
}