using System;
using System.Collections.Generic;
using System.Text;
using System.Workflow.Activities;
using System.Workflow.Runtime;
using System.Data;

namespace SodaService
{
    [Serializable]
    public class WorkflowSodaMachineDataService
    {
        protected static WorkflowRuntime _workflowRuntime = null;
        protected static ExternalDataExchangeService _dataExchangeService = null;
        protected static SodaMachineDataConnector _dataConnector = null;
        protected static object _syncRoot = new object();

        public event EventHandler<TotalChangedEventArgs> TotalChanged;
        public event EventHandler<DispenseEventArgs> Dispense;

        private Guid _instanceID = Guid.Empty;

        public Guid InstanceID
        {
            get { return _instanceID; }
            set { _instanceID = value; }
        }

        public static WorkflowSodaMachineDataService CreateDataService(Guid instanceID, WorkflowRuntime workflowRuntime)
        {
            lock (_syncRoot)
            {
                // If we're just starting, save a copy of the workflow runtime reference
                if (_workflowRuntime == null)
                {
                    _workflowRuntime = workflowRuntime;
                } // if

                // If we're just starting, plug in ExternalDataExchange service
                if (_dataExchangeService == null)
                {
                    _dataExchangeService = new ExternalDataExchangeService();
                    _workflowRuntime.AddService(_dataExchangeService);
                } // if

                // Check to see if we have already added this data exchange service
                SodaMachineDataConnector dataConnector = (SodaMachineDataConnector)workflowRuntime.GetService(typeof(SodaMachineDataConnector));
                if (dataConnector == null)
                {
                    _dataConnector = new SodaMachineDataConnector();
                    _dataExchangeService.AddService(_dataConnector);
                } // if
                else
                {
                    _dataConnector = dataConnector;
                } // else

                // Pull the service instance we registered with the connection object
                WorkflowSodaMachineDataService workflowDataService = new WorkflowSodaMachineDataService(instanceID);
                SodaMachineDataConnector.SodaMachineDataService = workflowDataService;

                return workflowDataService;
            } // lock
        }

        public static WorkflowSodaMachineDataService GetRegisteredWorkflowDataService()
        {
            lock (_syncRoot)
            {
                WorkflowSodaMachineDataService workflowDataService = SodaMachineDataConnector.SodaMachineDataService;

                if (workflowDataService == null)
                {
                    throw new Exception("Error configuring data service...service cannot be null.");
                } // if

                return workflowDataService;
            } // lock
        }

        private WorkflowSodaMachineDataService(Guid instanceID)
        {
            _instanceID = instanceID;
            SodaMachineDataConnector.SodaMachineDataService = this;
        }

        ~WorkflowSodaMachineDataService()
        {
            // Clean up
            _workflowRuntime = null;
            _dataExchangeService = null;
            _dataConnector = null;
        }
            
        public decimal Read()
        {
            return _dataConnector.Total;
        }

        public void RaiseTotalChangedEvent(decimal total)
        {
            if (_workflowRuntime == null)
                _workflowRuntime = new WorkflowRuntime();

            _workflowRuntime.GetWorkflow(_instanceID); // loads persisted workflow instances
            if (TotalChanged != null)
            {
                TotalChanged(this, new TotalChangedEventArgs(_instanceID, total));
            } // if
        }

        public void RaiseDispenseEvent(decimal finalTotal)
        {
            if (_workflowRuntime == null)
                _workflowRuntime = new WorkflowRuntime();

            _workflowRuntime.GetWorkflow(_instanceID); // loads persisted workflow instances
            if (Dispense != null)
            {
                Dispense(this, new DispenseEventArgs(_instanceID, finalTotal));
            } // if
        }
    }
}
