using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Workflow.Activities;
using System.Workflow.Runtime;
using System.Data;

namespace SodaService
{
    [Serializable]
    public class SodaMachineDataConnector : ISodaController 
    {
        protected decimal _dataValue = 0.0m;
        protected static WorkflowSodaMachineDataService _service = null;
        protected static object _syncLock = new object();

        public static WorkflowSodaMachineDataService SodaMachineDataService
        {
            get { return _service; }
            set
            {
                lock (_syncLock)
                {
                    _service = value;
                } // lock
            }
        }

        public decimal Total
        {
            get { return _dataValue; }
        }

        public void UpdateTotal(decimal total)
        {
            // Assign the field for later recall
            _dataValue = Total;

            // Raise the event to trigger host read
            _service.RaiseTotalChangedEvent(total);
        }

        public void ReadyToDispense(decimal finalTotal)
        {
            // Raise the event to trigger host action
            _service.RaiseDispenseEvent(finalTotal);
        }

        // Host to workflow events
        public event EventHandler<CoinInsertedEventArgs> CoinInserted;

        public void RaiseCoinInserted(Guid instanceID, decimal coinValue)
        {
            if (CoinInserted != null)
            {
                // Fire event
                CoinInserted(null, new CoinInsertedEventArgs(instanceID, coinValue));
            } // if
        }

        public event EventHandler<ItemSelectedEventArgs> ItemSelected;

        public void RaiseItemSelected(Guid instanceID)
        {
            if (ItemSelected != null)
            {
                // Fire event
                ItemSelected(null, new ItemSelectedEventArgs(instanceID));
            } // if
        }
    }
}
